// aboutdlg.cpp : implementation of the CAboutDlg class
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"

#include "paramdlg.h"

LRESULT CParamDlg::OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	m_refreshing = false;

	CRect rc, crc;
	CWindow parent;
	parent = GetParent();
	parent.GetClientRect(&rc);
	parent.ClientToScreen(&rc);
	GetWindowRect(&crc);
	CPoint pt = rc.TopLeft() + CPoint(rc.Size().cx, 0);
	pt.Offset(CSize(-crc.Size().cx, 0));
	rc = CRect(pt, crc.Size());
	SetWindowPos(NULL, rc, SWP_SHOWWINDOW);

	// note: must be same order of enum BPP
	CComboBox cbx_bpp(GetDlgItem(IDC_BPP));
	cbx_bpp.AddString(TEXT("1"));
	cbx_bpp.AddString(TEXT("4"));
	cbx_bpp.AddString(TEXT("8"));
	cbx_bpp.AddString(TEXT("16"));
	cbx_bpp.AddString(TEXT("24"));
	cbx_bpp.AddString(TEXT("32"));

	return TRUE;
}

LRESULT CParamDlg::OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	m_refreshing = true; // avoid bugs...
	return TRUE;
}

LRESULT CParamDlg::OnChanged(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	storeAndNotify();
	return TRUE;
}

LRESULT CParamDlg::OnWidthHeightSliderChanged(WORD wPosition, WORD wCode, HWND hWndCtl, BOOL& /*bHandled*/)
{
	// don't disturb while refreshing
	if(m_refreshing)
	{
		return TRUE;
	}

	int idc = (hWndCtl == GetDlgItem(IDC_WIDTHSLIDER)) ? IDC_WIDTH : IDC_HEIGHT;
	int idcslider = (idc == IDC_WIDTH) ? IDC_WIDTHSLIDER : IDC_HEIGHTSLIDER;

	if(!(wCode == TB_THUMBTRACK || wCode == TB_THUMBPOSITION))
	{
		wPosition = CTrackBarCtrl(GetDlgItem(idcslider)).GetPos();
	}

	LARGE_INTEGER li;
	m_pDoc->filesize(&li);
	int m = static_cast<int>(min(li.QuadPart, SHORT_MAX));

	// TODO: don't use linear, use exp.
	SetDlgItemInt(idc, static_cast<UINT>(static_cast<double>(wPosition) / m * li.QuadPart), FALSE);
	storeAndNotify();
	return TRUE;
}

void CParamDlg::setViewAndDoc(HWND hWndView, std::shared_ptr<CRawDoc> pDoc)
{
	m_hWndView = hWndView;
	m_pDoc = pDoc;

	if(!m_pDoc)
	{
		return;
	}

	m_refreshing = true;

	SetDlgItemInt(IDC_OFFSET, m_pDoc->m_offset, FALSE);

	SetDlgItemInt(IDC_WIDTH, m_pDoc->m_width, FALSE);

	SetDlgItemInt(IDC_HEIGHT, m_pDoc->m_height, FALSE);

	CButton(GetDlgItem(IDC_INVERTY)).SetCheck(m_pDoc->m_inverty);

	refreshTrackbars();

	CComboBox cbx_bpp(GetDlgItem(IDC_BPP));
	cbx_bpp.SetCurSel(m_pDoc->m_bpp); // XXX: direct enum BPP to int!

	refreshOrigs();

	CButton(GetDlgItem(IDC_ENDIANESS)).SetCheck(m_pDoc->m_endianess);

	SetDlgItemInt(IDC_LINEPAD, m_pDoc->m_linepad, FALSE);

	m_refreshing = false;
}

void CParamDlg::refreshUpDown(int spinid, int textid, unsigned int minint, unsigned int maxint)
{
	CUpDownCtrl ud(GetDlgItem(spinid));
	ud.SetRange32(minint, maxint);
	ud.SetPos32(GetDlgItemInt(textid));
}

void CParamDlg::refreshTrackbars()
{
	LARGE_INTEGER li;
	m_pDoc->filesize(&li);

	unsigned int mshort = static_cast<unsigned int>(min(li.QuadPart, SHORT_MAX));
	unsigned int mint = static_cast<unsigned int>(min(li.QuadPart, INT_MAX));

	refreshUpDown(IDC_OFFSETSPIN, IDC_OFFSET, 0, mint);
	refreshUpDown(IDC_WIDTHSPIN, IDC_WIDTH, 1, mint);
	refreshUpDown(IDC_HEIGHTSPIN, IDC_HEIGHT, 1, mint);
	refreshUpDown(IDC_LINEPADSPIN, IDC_LINEPAD, 0, mint);

	{
		CTrackBarCtrl tbw(GetDlgItem(IDC_WIDTHSLIDER));
		tbw.SetRange(1, mshort, TRUE); // XXX: uint...?
		tbw.SetPos(static_cast<int>(static_cast<double>(m_pDoc->m_width) * mshort / li.QuadPart));
	}

	{
		CTrackBarCtrl tbh(GetDlgItem(IDC_HEIGHTSLIDER));
		tbh.SetRange(1, mshort, TRUE); // XXX: uint...?
		tbh.SetPos(static_cast<int>(static_cast<double>(m_pDoc->m_height) * mshort / li.QuadPart));
	}
}

struct order CParamDlg::getRGBAOrder() const
{
	struct order order_rgba(0, 8, 16, 24);
	if(m_pDoc->m_bpp == BPP_16)
	{
		order_rgba.r = 0;
		order_rgba.g = m_pDoc->m_field16.r;
		order_rgba.b = order_rgba.g + m_pDoc->m_field16.g;
		order_rgba.a = order_rgba.b + m_pDoc->m_field16.b;
	}
	return order_rgba;
}

void CParamDlg::refreshOrigs()
{
	{
		unsigned int idc_rgb;
		if(CButton(GetDlgItem(IDC_RGBAORIG)).GetCheck()) { idc_rgb = IDC_RGBAORIG; }
		else if(m_pDoc->m_field16.equals(5, 6, 5, 0)) { idc_rgb = IDC_RGB565; }
		else if(m_pDoc->m_field16.equals(5, 5, 5, 0)) { idc_rgb = IDC_RGB555; }
		else if(m_pDoc->m_field16.equals(5, 5, 5, 1)) { idc_rgb = IDC_RGBA5551; }
		else if(m_pDoc->m_field16.equals(4, 4, 4, 4)) { idc_rgb = IDC_RGBA4444; }
		else { idc_rgb = IDC_RGBAORIG; }
		SendDlgItemMessage(idc_rgb, BM_SETCHECK, BST_CHECKED, 0);
		// XXX: assuming BOOL TRUE/FALSE == bool true/false
		CWindow(GetDlgItem(IDC_RGBAORIGR)).EnableWindow(idc_rgb == IDC_RGBAORIG);
		CWindow(GetDlgItem(IDC_RGBAORIGG)).EnableWindow(idc_rgb == IDC_RGBAORIG);
		CWindow(GetDlgItem(IDC_RGBAORIGB)).EnableWindow(idc_rgb == IDC_RGBAORIG);
		CWindow(GetDlgItem(IDC_RGBAORIGA)).EnableWindow(idc_rgb == IDC_RGBAORIG);
		if(idc_rgb != IDC_RGBAORIG)
		{
			SetDlgItemInt(IDC_RGBAORIGR, m_pDoc->m_field16.r, FALSE);
			SetDlgItemInt(IDC_RGBAORIGG, m_pDoc->m_field16.g, FALSE);
			SetDlgItemInt(IDC_RGBAORIGB, m_pDoc->m_field16.b, FALSE);
			SetDlgItemInt(IDC_RGBAORIGA, m_pDoc->m_field16.a, FALSE);
		}
	}

	{
		unsigned int idc_order;
		struct order order_rgba = getRGBAOrder();
		if(CButton(GetDlgItem(IDC_ORDERORIG)).GetCheck()) { idc_order = IDC_ORDERORIG; }
		else if(m_pDoc->m_order.equals(order_rgba)) { idc_order = IDC_ORDERRGBA; }
		else if(m_pDoc->m_order.equals(order_rgba.bgra())) { idc_order = IDC_ORDERBGRA; }
		else { idc_order = IDC_ORDERORIG; }
		SendDlgItemMessage(idc_order, BM_SETCHECK, BST_CHECKED, 0);
		// XXX: assuming BOOL TRUE/FALSE == bool true/false
		CWindow(GetDlgItem(IDC_ORDERORIGR)).EnableWindow(idc_order == IDC_ORDERORIG);
		CWindow(GetDlgItem(IDC_ORDERORIGG)).EnableWindow(idc_order == IDC_ORDERORIG);
		CWindow(GetDlgItem(IDC_ORDERORIGB)).EnableWindow(idc_order == IDC_ORDERORIG);
		CWindow(GetDlgItem(IDC_ORDERORIGA)).EnableWindow(idc_order == IDC_ORDERORIG);
		if(idc_order != IDC_ORDERORIG)
		{
			SetDlgItemInt(IDC_ORDERORIGR, m_pDoc->m_order.r, FALSE);
			SetDlgItemInt(IDC_ORDERORIGG, m_pDoc->m_order.g, FALSE);
			SetDlgItemInt(IDC_ORDERORIGB, m_pDoc->m_order.b, FALSE);
			SetDlgItemInt(IDC_ORDERORIGA, m_pDoc->m_order.a, FALSE);
		}
	} 
}

void CParamDlg::storeAndNotify()
{
	// FIXME: multi-threading?
	if(m_refreshing)
	{
		return;
	}

	m_refreshing = true;

	m_pDoc->m_offset = GetDlgItemInt(IDC_OFFSET, NULL, FALSE);

	m_pDoc->m_width = max(1, GetDlgItemInt(IDC_WIDTH, NULL, FALSE));

	m_pDoc->m_height = max(1, GetDlgItemInt(IDC_HEIGHT, NULL, FALSE));

	m_pDoc->m_inverty = CButton(GetDlgItem(IDC_INVERTY)).GetCheck() == BST_CHECKED;

	refreshTrackbars();

	CComboBox cbx_bpp(GetDlgItem(IDC_BPP));
	m_pDoc->m_bpp = static_cast<enum BPP>(cbx_bpp.GetCurSel()); // XXX: direct int to enum BPP!

	if(CButton(GetDlgItem(IDC_RGB565)).GetCheck()) { m_pDoc->m_field16.set(5, 6, 5, 0); }
	else if(CButton(GetDlgItem(IDC_RGB555)).GetCheck()) { m_pDoc->m_field16.set(5, 5, 5, 0); }
	else if(CButton(GetDlgItem(IDC_RGBA5551)).GetCheck()) { m_pDoc->m_field16.set(5, 5, 5, 1); }
	else if(CButton(GetDlgItem(IDC_RGBA4444)).GetCheck()) { m_pDoc->m_field16.set(4, 4, 4, 4); }
	else
	{
		m_pDoc->m_field16.r = GetDlgItemInt(IDC_RGBAORIGR);
		m_pDoc->m_field16.g = GetDlgItemInt(IDC_RGBAORIGG);
		m_pDoc->m_field16.b = GetDlgItemInt(IDC_RGBAORIGB);
		m_pDoc->m_field16.a = GetDlgItemInt(IDC_RGBAORIGA);
	}

	struct order order_rgba = getRGBAOrder();
	if(CButton(GetDlgItem(IDC_ORDERRGBA)).GetCheck()) { m_pDoc->m_order = order_rgba; }
	else if(CButton(GetDlgItem(IDC_ORDERBGRA)).GetCheck()) { m_pDoc->m_order.set(order_rgba.bgra()); }
	else
	{
		m_pDoc->m_order.r = GetDlgItemInt(IDC_ORDERORIGR);
		m_pDoc->m_order.g = GetDlgItemInt(IDC_ORDERORIGG);
		m_pDoc->m_order.b = GetDlgItemInt(IDC_ORDERORIGB);
		m_pDoc->m_order.a = GetDlgItemInt(IDC_ORDERORIGA);
	}

	m_pDoc->m_endianess = CButton(GetDlgItem(IDC_ENDIANESS)).GetCheck() == BST_CHECKED;

	m_pDoc->m_linepad = GetDlgItemInt(IDC_LINEPAD, NULL, FALSE);

	refreshOrigs();

	::SendMessage(m_hWndView, WM_APP + 0, 0, 0);

	m_refreshing = false;
}
