// rawviewView.cpp : implementation of the CRawviewView class
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"

#include "rawviewView.h"

BOOL CRawviewView::PreTranslateMessage(MSG* pMsg)
{
	pMsg;
	return FALSE;
}

void CRawviewView::fillbits(unsigned char* pbits) const
{
	const HANDLE hFile = m_pDoc->getFileHandle();

	LARGE_INTEGER limax;
	m_pDoc->filesize(&limax);

	LARGE_INTEGER lipos;
	lipos.QuadPart = m_pDoc->m_offset;
	if(!SetFilePointerEx(hFile, lipos, NULL, FILE_BEGIN))
	{
		return;
	}

	unsigned int x = 0, y = m_pDoc->m_height - 1;
	unsigned int bytepercount, countperbyte;
	switch(m_pDoc->m_bpp)
	{
	case BPP_1: bytepercount = 1; countperbyte = 8; break;
	case BPP_4: bytepercount = 1; countperbyte = 2; break;
	case BPP_8: bytepercount = 1; countperbyte = 1; break;
	case BPP_16: bytepercount = 2; countperbyte = 1; break;
	case BPP_24: bytepercount = 3; countperbyte = 1; break;
	case BPP_32: bytepercount = 4; countperbyte = 1; break;
	default:
		return;
	}
	while(lipos.QuadPart < limax.QuadPart)
	{
		DWORD read;
		char buf[4];

		if(!ReadFile(hFile, buf, bytepercount, &read, NULL))
		{
			return;
		}
		if(read != bytepercount)
		{
			// should be EOF
			return;
		}

		lipos.QuadPart += read;

		for(unsigned int i = 0; i < countperbyte; i++)
		{
			unsigned int yy = y;
			if(m_pDoc->m_inverty)
			{
				yy = m_pDoc->m_height - 1 - y;
			}
			unsigned char *p = pbits + (yy * m_pDoc->m_width + x) * sizeof(DWORD);

			switch(m_pDoc->m_bpp)
			{
			case BPP_1:
				if(m_pDoc->m_endianess == FALSE)
				{
					p[0] = p[1] = p[2] = ((buf[0] >> i) & 0x01) ? 255 : 0; // B/G/R
				}
				else
				{
					p[0] = p[1] = p[2] = ((buf[0] << i) & 0x80) ? 255 : 0; // B/G/R
				}
				p[3] = 0; // x
				break;
			case BPP_4:
				if(m_pDoc->m_endianess == FALSE)
				{
					p[0] = p[1] = p[2] = (((buf[0] << (i * 4)) & 0xF0) << 0); // B/G/R
				}
				else
				{
					p[0] = p[1] = p[2] = (((buf[0] >> (i * 4)) & 0x0F) << 4); // B/G/R
				}
				p[0] = p[1] = p[2] = p[0] | (p[0] >> 4); // ex. 0x00->0x00, 0x20->0x22, 0xF0->0xFF
				p[3] = 0; // x
				break;
			case BPP_8:
				p[0] = buf[0]; // B
				p[1] = buf[0]; // G
				p[2] = buf[0]; // R
				p[3] = 0; // x
				break;
			case BPP_16:
				{
					unsigned short pp = *reinterpret_cast<unsigned short *>(buf);
					// TODO: field=3bit value=0b111 become 0xE0 instead of 0xFF
					p[0] = ((static_cast<unsigned int>(pp) >> (m_pDoc->m_order.b)) & (1 << (m_pDoc->m_field16.b)) - 1) << (8 - m_pDoc->m_field16.b); // B
					p[1] = ((static_cast<unsigned int>(pp) >> (m_pDoc->m_order.g)) & (1 << (m_pDoc->m_field16.g)) - 1) << (8 - m_pDoc->m_field16.g); // G
					p[2] = ((static_cast<unsigned int>(pp) >> (m_pDoc->m_order.r)) & (1 << (m_pDoc->m_field16.r)) - 1) << (8 - m_pDoc->m_field16.r); // R
					p[3] = 0; // x
					break;
				}
			case BPP_24: /* FALLTHROUGH */
			case BPP_32:
				{
					unsigned int pp = *reinterpret_cast<unsigned int *>(buf);
					p[0] = (pp >> m_pDoc->m_order.b) & 0xFF; // B
					p[1] = (pp >> m_pDoc->m_order.g) & 0xFF; // G
					p[2] = (pp >> m_pDoc->m_order.r) & 0xFF; // R
					p[3] = 0; // x TODO: A
					break;
				}
			}

			x++;
			if(m_pDoc->m_width <= x)
			{
				x = 0;
				if(y <= 0) // y is unsigned; < 0 is never happen...
				{
					goto quit;
				}
				y--;
				LARGE_INTEGER lidelta;
				lidelta.QuadPart = m_pDoc->m_linepad;
				if(!SetFilePointerEx(hFile, lidelta, NULL, FILE_CURRENT))
				{
					return;
				}
				lipos.QuadPart += lidelta.QuadPart;
				break;
			}
		}
	}
quit:
	;
}

void CRawviewView::updateCache(CDCHandle dc)
{
	CDC bdc(CreateCompatibleDC(dc));
	if(bdc == NULL)
	{
		return;
	}

	BITMAPINFO bmi = {0};
	unsigned char *pbits;

	bmi.bmiHeader.biSize = sizeof(bmi.bmiHeader);
	bmi.bmiHeader.biWidth = m_pDoc->m_width;
	bmi.bmiHeader.biHeight = m_pDoc->m_height;
	bmi.bmiHeader.biPlanes = 1;
	bmi.bmiHeader.biBitCount = 32; // is easy to align
	bmi.bmiHeader.biCompression = BI_RGB;
	bmi.bmiHeader.biSizeImage = 0; // maybe 0 when BI_RGB

	CBitmap hbm(CreateDIBSection(bdc, &bmi, DIB_RGB_COLORS, reinterpret_cast<void**>(&pbits), NULL, 0));
	if(hbm == NULL)
	{
		return;
	}

	fillbits(pbits);

	DeleteObject(bdc.SelectBitmap(hbm));

	m_hCacheDC = bdc.Detach();
}

void CRawviewView::DoPaint(CDCHandle dc)
{
	dc.FillSolidRect(CRect(CPoint(0, 0), m_sizeClient), GetSysColor(COLOR_WINDOW));

	if(m_pDoc)
	{
		if(m_hCacheDC == NULL)
		{
			updateCache(dc);
		}
		dc.BitBlt(0, 0, m_pDoc->m_width, m_pDoc->m_height, m_hCacheDC, 0, 0, SRCCOPY);
	}
}

LRESULT CRawviewView::OnParamDlgChanged(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	SetScrollSize(m_pDoc->m_width, m_pDoc->m_height, TRUE, FALSE); // note: size may be changed
	m_hCacheDC = NULL; // invalidate cache
	return 0;
}

void CRawviewView::setDocument(std::shared_ptr<CRawDoc> pDoc)
{
	m_pDoc = pDoc;

	if(m_paramDlg.m_hWnd)
	{
		m_paramDlg.DestroyWindow();
	}
	m_hCacheDC = NULL; // invalidate cache

	if(m_pDoc)
	{
		m_paramDlg.Create(m_hWnd);
	}
	m_paramDlg.setViewAndDoc(m_hWnd, m_pDoc);
	if(m_pDoc)
	{
		SetScrollSize(m_pDoc->m_width, m_pDoc->m_height, TRUE, FALSE); // note: size may be changed
	}

	Invalidate();
}
